#ifndef __MC_HID_SERVER_HPP
#define __MC_HID_SERVER_HPP

#include <stdint.h>
#include <dbus-c++/dbus.h>
#include <boost/signals2.hpp>

#include "mc-hid-server-glue.hpp"

#include "gpio/gpio.hpp"
#include "mcp23017/mcp23017.hpp"
#include "thread/thread.hpp"
#include "buttontimer/buttontimer.hpp"

class HidServer 
  : public nl::miqra::MediaCore::Hid_adaptor, // << This will be generated by the makefile using dbusxx-xml2cpp on mc-hid-introspect.xml
  public DBus::IntrospectableAdaptor,
  public DBus::ObjectAdaptor,
  public Thread
{
public:

    HidServer(DBus::Connection &connection);
    ~HidServer();
    
    virtual void SetColor(const uint8_t& r, const uint8_t& g, const uint8_t& b);
    virtual void PulseColor(const uint8_t& r, const uint8_t& g, const uint8_t& b, const double& interval);
    virtual void ClearColor();
    virtual bool JackState();

    void onInterrupt(GpioPin * sender, GpioEdge edge, bool pinval);
    void onInterruptError(Thread * sender, ThreadException x);
    
    bool onValidatePress(uint16_t keycode);
    void onShortPress(uint16_t keycode);
    void onLongPress(uint16_t keycode);

protected:
	virtual void ThreadLoop();

private:
    Mcp23017 *mcp;
    GpioPin *intpin;
    ButtonTimer *btnTimer;
    
    int64_t noiseTimeout; 
    
	uint8_t pulseR, pulseG, pulseB,pulseI,pulseMin;
    int32_t pulseTime;
	bool pulseDirUp;
	
    void initHardware(void);
    
    void keyUp(uint16_t keycode);
    void keyDown(uint16_t keycode);
	std::string getBtnName(uint16_t keycode);
    
    boost::signals2::connection onInterruptConnection;
	boost::signals2::connection onInterruptErrorConnection;
    boost::signals2::connection onShortPressConnection;
    boost::signals2::connection onLongPressConnection;
    boost::signals2::connection onValidatePressConnection;
};

#endif//__MC_HID_SERVER_HPP